/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Render, a 3-address code Java(TM) bytecode representation.        *
 * Copyright (C) 1996-1998 Raja Vallee-Rai (kor@sable.mcgill.ca)    *
 * All rights reserved.                                              *
 *                                                                   *
 * This work is free software; you can redistribute it and/or        *
 * modify it under the terms of the GNU Library General Public       *
 * License as published by the Free Software Foundation; either      *
 * version 2 of the License, or (at your option) any later version.  *
 *                                                                   *
 * This work is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU *
 * Library General Public License for more details.                  *
 *                                                                   *
 * You should have received a copy of the GNU Library General Public *
 * License along with this library; if not, write to the             *
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,      *
 * Boston, MA  02111-1307, USA.                                      *
 *                                                                   *
 * Java is a trademark of Sun Microsystems, Inc.                     *
 *                                                                   *
 * To submit a bug report, send a comment, or get the latest news on *
 * this project and other Sable Research Group projects, please      *
 * visit the web site: http://www.sable.mcgill.ca/                   *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/*
 Change History
 --------------
 A) Notes:

 Please use the following template.  Most recent changes should
 appear at the top of the list.

 - Modified on [date (March 1, 1900)] by [name]. [(*) if appropriate]
   [description of modification].

 Any Modification flagged with "(*)" was done as a project of the
 Sable Research Group, School of Computer Science,
 McGill University, Canada (http://www.sable.mcgill.ca/).

 You should add your copyright, using the following template, at
 the top of this file, along with other copyrights.

 *                                                                   *
 * Modifications by [name] are                                       *
 * Copyright (C) [year(s)] [your name (or company)].  All rights     *
 * reserved.                                                         *
 *                                                                   *

 B) Changes:

 - Modified on September 25, 1998 by Raja Vallee-Rai (kor@sable.mcgill.ca) (*)
   Changed the license.
    
 - Modified on October, 1996 by Raja Vallee-Rai 
   First release.
*/

public class Basis3D
{
	static final int NUM_ROTATIONS_UNTIL_CORRECTION = 15;
	
	Vector3D x, y, z;
	int numRotationsSinceCorrection;
		
	public Basis3D()
	{
		x = new Vector3D(1, 0, 0);
		y = new Vector3D(0, 1, 0);
		z = new Vector3D(0, 0, 1);
	}
	
	public void rotateAroundAxis(Vector3D axis, double angle)
	{
		double a11, a12, a13;
	    double a21, a22, a23;
    	double a31, a32, a33;
        
    	if(axis.z == -1)
		{
        	axis.z = 1;
        	angle = -angle;			
    	}

    	// Construct rotation matrix.
    	{
        	double squaredX, squaredY, squaredZ;
        	double oneMinusCos;
        	double xy, xz, yz;
        	double sinOfAngle, cosOfAngle;
        
       	 	squaredX = axis.x * axis.x;
        	squaredY = axis.y * axis.y;
        	squaredZ = axis.z * axis.z;
    
        	sinOfAngle = Math.sin(-angle);
        	cosOfAngle = Math.cos(-angle);
    
        	oneMinusCos = (1 - cosOfAngle);
        
        	xy = axis.x * axis.y;
        	xz = axis.x * axis.z;
        	yz = axis.y * axis.z;
        
        	a11 = squaredX + cosOfAngle * (1 - squaredX);
        	a22 = squaredY + cosOfAngle * (1 - squaredY);
        	a33 = squaredZ + cosOfAngle * (1 - squaredZ);
		
        	// Compute a12 and a21.
        	{
            	double XY_x_OneMinusCos;
        	    double axisZ_x_SinOfAngle;
            
            	XY_x_OneMinusCos = xy * oneMinusCos;
            	axisZ_x_SinOfAngle = axis.z * sinOfAngle;
            
            	a12 = XY_x_OneMinusCos - axisZ_x_SinOfAngle;
            	a21 = XY_x_OneMinusCos + axisZ_x_SinOfAngle;
        	}
        
	        // Compute a13 and a31.
    	    {
        	    double XZ_x_OneMinusCos;
            	double axisY_x_SinOfAngle;
            
            	XZ_x_OneMinusCos = xz * oneMinusCos;
            	axisY_x_SinOfAngle = axis.y * sinOfAngle;
            
            	a13 = XZ_x_OneMinusCos + axisY_x_SinOfAngle;
            	a31 = XZ_x_OneMinusCos - axisY_x_SinOfAngle;
	        }
                
    	    // Compute a23 and a32.
        	{
	            double YZ_x_OneMinusCos;
   	         	double axisX_x_SinOfAngle;
            
            	YZ_x_OneMinusCos = yz * oneMinusCos;
            	axisX_x_SinOfAngle = axis.x * sinOfAngle;
            
            	a23 = YZ_x_OneMinusCos - axisX_x_SinOfAngle;
            	a32 = YZ_x_OneMinusCos + axisX_x_SinOfAngle;
        	}
    	}
		  
	    // Rotate every vector of the space basis.
    	{
			// for the x vector of the basis
			{  
            	double X, Y, Z;
            	double Tx, Ty, Tz;
            
            	X = x.x;
				Y = x.y;
            	Z = x.z;
            
            	Tx = a11 * X +
                	 a12 * Y +
                 	 a13 * Z;
                 
            	Ty = a21 * X +
                	 a22 * Y +
                 	 a23 * Z;

            	Tz = a31 * X +
                 	 a32 * Y +
                 	 a33 * Z;

				x.x = Tx;
				x.y = Ty;
				x.z = Tz;
        	}

			// for the y vector of the basis
			{  
            	double X, Y, Z;
            	double Tx, Ty, Tz;
            
            	X = y.x;
				Y = y.y;
            	Z = y.z;
            
            	Tx = a11 * X +
                	 a12 * Y +
                 	 a13 * Z;
                 
            	Ty = a21 * X +
                 	 a22 * Y +
                 	 a23 * Z;

            	Tz = a31 * X +
                 	 a32 * Y +
                 	 a33 * Z;

				y.x = Tx;
				y.y = Ty;
				y.z = Tz;
        	}
		
			// for the x vector of the basis
			{  
            	double X, Y, Z;
            	double Tx, Ty, Tz;
            
            	X = z.x;
				Y = z.y;
            	Z = z.z;
            
            	Tx = a11 * X +
                	 a12 * Y +
 					 a13 * Z;
                 
            	Ty = a21 * X +
                 	 a22 * Y +
                 	 a23 * Z;

            	Tz = a31 * X +
                 	 a32 * Y +
                     a33 * Z;

				z.x = Tx;
				z.y = Ty;
				z.z = Tz;
        	}
		}

	}
    
    public String toString()
    {
        return "[" + x.toString() + "," + y.toString() + "," +
            z.toString() + "]";
    }
}


