/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Render, a 3-address code Java(TM) bytecode representation.        *
 * Copyright (C) 1996-1998 Raja Vallee-Rai (kor@sable.mcgill.ca)    *
 * All rights reserved.                                              *
 *                                                                   *
 * This work is free software; you can redistribute it and/or        *
 * modify it under the terms of the GNU Library General Public       *
 * License as published by the Free Software Foundation; either      *
 * version 2 of the License, or (at your option) any later version.  *
 *                                                                   *
 * This work is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU *
 * Library General Public License for more details.                  *
 *                                                                   *
 * You should have received a copy of the GNU Library General Public *
 * License along with this library; if not, write to the             *
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,      *
 * Boston, MA  02111-1307, USA.                                      *
 *                                                                   *
 * Java is a trademark of Sun Microsystems, Inc.                     *
 *                                                                   *
 * To submit a bug report, send a comment, or get the latest news on *
 * this project and other Sable Research Group projects, please      *
 * visit the web site: http://www.sable.mcgill.ca/                   *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/*
 Change History
 --------------
 A) Notes:

 Please use the following template.  Most recent changes should
 appear at the top of the list.

 - Modified on [date (March 1, 1900)] by [name]. [(*) if appropriate]
   [description of modification].

 Any Modification flagged with "(*)" was done as a project of the
 Sable Research Group, School of Computer Science,
 McGill University, Canada (http://www.sable.mcgill.ca/).

 You should add your copyright, using the following template, at
 the top of this file, along with other copyrights.

 *                                                                   *
 * Modifications by [name] are                                       *
 * Copyright (C) [year(s)] [your name (or company)].  All rights     *
 * reserved.                                                         *
 *                                                                   *

 B) Changes:

 - Modified on September 25, 1998 by Raja Vallee-Rai (kor@sable.mcgill.ca) (*)
   Changed the license.
    
 - Modified on October, 1996 by Raja Vallee-Rai 
   First release.
*/

/****************************************************

    Math3D.java

    3D math functions

    by Raja Vallee-Rai(1-Oct-96)
		kor@cs.mcgill.ca
 ****************************************************/

public class Vector3D
	// Class which handles 3D vector mathematics.
	// I commented out the methods which allow you to specify
	// a vector3d implicitly by giving the components in the method arguments in
	// order to test out the OO paradigm.
{
    double x, y, z;
    
    public Vector3D(double x, double y, double z)
    {
        this.x = x;
        this.y = y;
		this.z = z;
    }

	public double magnitude()
	{
		return Math.sqrt(x*x + y*y + z*z);
	}
	
	public double magnitudeSquared()
	{
		return (x*x + y*y + z*z);
	}
	
    public Vector3D normalized()
    {
        double magnitude;
        
        magnitude = Math.sqrt(x*x + y*y + z*z);
        
        if(FuzzyMath.coreOf(magnitude) != 0.0)
            return new Vector3D(x / magnitude, y / magnitude, z / magnitude);
        else
            return new Vector3D(0, 0, 0);
    }
    
	public void normalize()
    {
        double magnitude;
        
        magnitude = Math.sqrt(x*x + y*y + z*z);
        
        if(FuzzyMath.coreOf(magnitude) != 0.0)
		{
			x = x / magnitude;
			y = y / magnitude;
			z = z / magnitude;
		}
		else {
			x = 0;
			y = 0;
			z = 0;
		}
    }
    
    public void addOn(Vector3D other)	
		// performs this += other 
    {
        x += other.x;
        y += other.y;
		z += other.z;
    }
    
	/*
    public void addOn(double otherX, double otherY, double otherZ)
    {
        x += otherX;
        y += otherY;
		z += otherZ;
    } */
    
    public void substractOff(Vector3D other)
		// performs this -= other
    {
        x -= other.x;
        y -= other.y;
		z -= other.z;
    }
 
	/*   
    public void substractOff(double otherX, double otherY, double otherZ)
    {
        x -= otherX;
        y -= otherY;
		z -= otherZ;
    }*/
    
    public void multiplyBy(double scalar)
    {
        x *= scalar;
        y *= scalar;
		z *= scalar;
    }

	public double dotProductWith(Vector3D other)
	{
		return x * other.x + y * other.y + z * other.z;
	}

	public Vector3D crossProductWith(Vector3D other)
	{
		return new Vector3D(this.y * other.z - other.y * this.z,
							-(this.x * other.z - other.x * this.z),
							this.x * other.y - other.x * this.y);
	}
		
    public Vector3D copy()
    {
        return new Vector3D(x, y, z);
    }
    
    public Vector3D times(double scalar)
    {
        return new Vector3D(x * scalar, y * scalar, z * scalar);
    }
    
    public Vector3D plus(Vector3D other)
    {
        return new Vector3D(x + other.x, y + other.y, z + other.z);
    }
    
    public Vector3D minus(Vector3D other)
    {
        return new Vector3D(x - other.x, y - other.y, z - other.z);
    }
	    
	public String toString()
	{
		return "[" + x + ", " + y + ", " + z + "]";
	}
}
